/* Win32Linux.cpp - Win32 API for Linux */

/* Copyright 2009, acontis technologies GmbH */

/***************************************************************************
 * DESCRIPTION
 * This module contains a subset of the Win32 API for Linux.
 *
 ***************************************************************************

0.00,12sep09,szf    written.
*/


/*******************************************************************************
*                                 INCLUDES
*/
#include <Win32Linux.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>


/*******************************************************************************
*                                 DEFINES
*/


/*******************************************************************************
*                                 MACROS
*/


/*******************************************************************************
*                                 GLOBALS
*/


/*******************************************************************************
*                                 LOCALS
*/


/*******************************************************************************
*                           FORWARD DECLARATIONS
*/



/*******************************************************************************
*                            GLOBAL FUNCTIONS
*/



/*******************************************************************************
*
* Get symbol from shared object library
*
*/
void*                 /* Return:  pointer to symbol, NULL it not found */
GetProcAddress
(HINSTANCE hInstRtosLib     /* I  library handle */
,const char* szSymbol       /* I  symbol name */
)
{
void* pvSymbol = NULL;
char* szErrorInfo;
ostream& OutputStream = cout;

    // Look for symbol in the shared lib
    pvSymbol = dlsym( hInstRtosLib, szSymbol );
    if( pvSymbol == NULL )
    {
        szErrorInfo = dlerror();
        if( szErrorInfo != NULL )
        {
            OutputStream <<  "dlsym returned error '" << szErrorInfo << "' when looking for symbol '" << szSymbol << "'\n";
            goto Exit;
        }
        else
        {
            // The symbol has been found but it is NULL
            OutputStream <<  "dlsym returned NULL pointer when looking for symbol '" << szSymbol << "'\n";
            goto Exit;
        }
    }
    Exit:
    return pvSymbol;
}

/*******************************************************************************
*
* CriticalSection implementation
*
*/
void InitializeCriticalSection( CRITICAL_SECTION* pCritSec )
{
    pthread_mutexattr_t MutexAttr;
    // Set the mutex as a recursive mutex
    pthread_mutexattr_settype( &MutexAttr, PTHREAD_MUTEX_RECURSIVE_NP );

    // create the muteIdx with the attributes set
    pthread_mutex_init( pCritSec, &MutexAttr );

    //After initializing the mutex, the thread attribute can be destroyed
    pthread_mutexattr_destroy( &MutexAttr );
}
void DeleteCriticalSection( CRITICAL_SECTION* pCritSec )
{
    pthread_mutex_destroy( pCritSec );
}
void EnterCriticalSection( CRITICAL_SECTION* pCritSec )
{
    pthread_mutex_lock( pCritSec );
}
void LeaveCriticalSection( CRITICAL_SECTION* pCritSec )
{
    pthread_mutex_unlock( pCritSec );
}

/*******************************************************************************
*
* Event implementation
*
*/
sem_t* CreateEvent( void* pDummy1, unsigned char bDummy2, unsigned char bDummy3, void* pDummy4 )
{
    sem_t* pSem;
    pSem = new sem_t;
    sem_init( pSem, 0, 0 );
    return pSem;
}
void DeleteEvent( sem_t* pSem )
{
    sem_destroy( pSem );
}
void SetEvent( sem_t* pSem )
{
    sem_post( pSem );
}
void ResetEvent( sem_t* pSem )
{
    sem_post( pSem );
}
unsigned int WaitForEvent( sem_t* pSem, UINT32 dwTimeout )
{
    ostream& OutputStream = cout;
    int nResult;
    struct timespec tsCurr;
    struct timespec tsLatest;
    UINT32 dwRetVal = WAIT_OBJECT_0;

    /* handle different timeout cases */
    switch (dwTimeout)
    {
    case 0:
        nResult = sem_trywait( pSem );
        if( (nResult == -1) && (errno == EAGAIN) )
        {
            dwRetVal = WAIT_TIMEOUT;
        }
        else
        {
            OutputStream << "WaitForEvent: sem_trywait returned an error\n";
            dwRetVal = WAIT_ABANDONED;
        }
        break;

    case (UINT32)-1:
        nResult = sem_wait( pSem );
        if( nResult != 0 )
        {
            OutputStream << "WaitForEvent: sem_wait returned an error\n";
            dwRetVal = WAIT_ABANDONED;
        }
        break;

    default:
        /* Calculate relative interval as current time plus number of milliseconds */
        if ( clock_gettime(CLOCK_REALTIME, &tsCurr) == -1 )
        {
            OutputStream << "WaitForEvent: clock_gettime returned an error\n";
        }
        tsLatest.tv_sec = tsCurr.tv_sec;
        if( dwTimeout > 1000000000 )
        {
            /* timeout >= 1 second, adjust seconds up front */
            tsLatest.tv_sec += dwTimeout/1000000000;
            dwTimeout = dwTimeout % 1000000000;
        }
        tsLatest.tv_nsec = tsCurr.tv_nsec + 1000000 * dwTimeout;
        if( tsLatest.tv_nsec < tsCurr.tv_nsec )
        {
            /* nanoseconds wrapped, add one additional second */
            tsLatest.tv_sec = tsCurr.tv_sec + 1;
        }
        nResult = sem_timedwait( pSem, &tsLatest );
        if( (nResult == -1) && (errno == ETIMEDOUT) )
        {
            dwRetVal = WAIT_TIMEOUT;
        }
        else if( nResult == -1 )
        {
            OutputStream << "WaitForEvent: sem_timedwait returned an error (" << errno << ")\n";
            dwRetVal = WAIT_ABANDONED;
        }
    }
    return dwRetVal;
}

/*******************************************************************************
*
* Thread implementation
*
*/
void* CreateThread
(void* pvThreadAttributes
,unsigned int dwStackSize
,void* pfThreadEntry
,void* pvParams
,unsigned int dwFlags
,unsigned int* pdwThreadId
)
{
    int nResult;
    pthread_t pThread;
    pthread_attr_t ThreadAttr;

    int nSchedPolicy = SCHED_RR;    // scheduling policy - real time
    //int nSchedPolicy = SCHED_FIFO;    // scheduling policy - FIFO
    struct sched_param SchedParam;  // scheduling priority
    void* pvThreadObject;

    /* set thread attributes */
    /*************************/
    pthread_attr_init( &ThreadAttr );
    /* Set the stack size of the thread */
    pthread_attr_setstacksize( &ThreadAttr, (dwStackSize==0)?0x10000:dwStackSize );
    /* Set thread to detached state. No need for pthread_join*/
    pthread_attr_setdetachstate( &ThreadAttr, PTHREAD_CREATE_DETACHED );
    /* priority is set statically */
    /* Set the policy of the thread to real time*/
    pthread_attr_setschedpolicy( &ThreadAttr, nSchedPolicy);
    /* Set the scheduling priority of the thread - (1 = lowest, 99 = highest) */
    SchedParam.sched_priority = 1;
    pthread_attr_setschedparam( &ThreadAttr, &SchedParam );

    /* create the thread */
    /*********************/
    nResult = pthread_create( &pThread, &ThreadAttr, (void*(*)(void*))pfThreadEntry, pvParams );
    if( nResult != 0 )
    {
        pvThreadObject = NULL;
        goto Exit;
    }
    pvThreadObject = (void*)pThread;


Exit:
    pthread_attr_destroy( &ThreadAttr );
    return pvThreadObject;

}
/*******************************************************************************
*
* Sleep implementation
*
*/
typedef enum
{
    eSLEEP_USLEEP = 1,
    eSLEEP_NANOSLEEP = 2,
    eSLEEP_CLOCK_NANOSLEEP = 3
}E_SLEEP;
static E_SLEEP S_ESleep = eSLEEP_CLOCK_NANOSLEEP;
void Sleep(unsigned int dwMsec)
{
    struct timespec ts;

    switch( S_ESleep )
    {
    case eSLEEP_USLEEP:
        usleep( 1000 * dwMsec );
        break;

    case eSLEEP_NANOSLEEP:
        ts.tv_sec = (dwMsec)/1000;
        ts.tv_nsec = 1000000 * ((dwMsec)%1000);
        nanosleep( &ts, NULL );
        break;

    case eSLEEP_CLOCK_NANOSLEEP:
    default:
        ts.tv_sec = (dwMsec)/1000;
        ts.tv_nsec = 1000000 * ((dwMsec)%1000);
        clock_nanosleep( CLOCK_REALTIME, 0, &ts, NULL );
        break;
    }
}


/*******************************************************************************
*
* File handling
*
*/
HANDLE CreateFile
(LPCTSTR lpFileName
,DWORD dwAccess
,DWORD dwShareMode
,LPSECURITY_ATTRIBUTES lpSec
,DWORD dwDisposition
,DWORD dwFlagsAndAttrib
,HANDLE hTemplate )
{
HANDLE hFile;
int nFlags = 0;

    if( dwAccess & GENERIC_READ )
    {
        if( dwAccess & GENERIC_WRITE )
        {
            nFlags = O_RDWR;
            nFlags |= O_CREAT;
        }
        else
        {
            nFlags = O_RDONLY;
        }
    }
    else if( dwAccess & GENERIC_WRITE )
    {
        nFlags = O_WRONLY;
        nFlags |= O_CREAT;
    }
    else
    {
        assert(FALSE);
    }
    if( dwDisposition == CREATE_ALWAYS )
    {
        nFlags |= O_TRUNC;
    }

    hFile = (HANDLE)open( lpFileName, nFlags );
    if( hFile == (HANDLE)-1 )
    {
        hFile = (HANDLE)INVALID_HANDLE_VALUE;
    }
    return hFile;
}
/*--------------------------*/
BOOL WriteFile( HANDLE hFile, LPVOID lpBuffer, DWORD dwNumBytes, DWORD* pdwBytesWritten, LPVOID pOverlapped )
{
BOOL bSuccess = TRUE;
ssize_t NumWritten;

    NumWritten = write( (int)hFile, lpBuffer, (size_t)dwNumBytes );
    if( NumWritten == -1 )
    {
        bSuccess = FALSE;
        goto Exit;
    }
    if( pdwBytesWritten != NULL )
    {
        *pdwBytesWritten = (DWORD)NumWritten;
    }
    Exit:
    return bSuccess;
}
/*--------------------------*/
BOOL WINAPI ReadFile
(HANDLE hFile
,LPVOID lpBuffer
,DWORD nNumberOfBytesToRead
,LPDWORD lpNumberOfBytesRead
,LPVOID lpOverlapped
)
{
BOOL bSuccess = TRUE;
ssize_t NumRead;

    NumRead = read( (int)hFile, lpBuffer, (size_t)nNumberOfBytesToRead );
    if( NumRead == -1 )
    {
        bSuccess = FALSE;
        goto Exit;
    }
    if( lpNumberOfBytesRead != (LPDWORD)NULL )
    {
        *lpNumberOfBytesRead = (DWORD)NumRead;
    }
    Exit:
    return bSuccess;
}
/*--------------------------*/
DWORD WINAPI SetFilePointer
(HANDLE hFile
,LONG lDistanceToMove
,PLONG lpDistanceToMoveHigh
,DWORD dwMoveMethod )
{
    off_t ResultOffset;
    int nWhence;
    DWORD dwNewOffset;

    switch( dwMoveMethod )
    {
    case FILE_END:
        nWhence = SEEK_END;
        break;
    case FILE_CURRENT:
        nWhence = SEEK_CUR;
        break;
    case FILE_BEGIN:
    default:
        nWhence = SEEK_SET;
        break;
    }
    ResultOffset = lseek( (int)hFile, (off_t)lDistanceToMove, nWhence );
    if( ResultOffset == (off_t)-1 )
    {
        dwNewOffset = INVALID_SET_FILE_POINTER;
        goto Exit;
    }
    dwNewOffset = (DWORD)ResultOffset;

    Exit:
    return dwNewOffset;
}

/*--------------------------*/
DWORD WINAPI GetFileSize(HANDLE hFile,LPDWORD lpFileSizeHigh)
{
int nResult;
DWORD dwResult;
struct stat Stat;

    nResult = fstat( (int)hFile, &Stat );
    if( nResult != 0 )
    {
        dwResult = INVALID_FILE_SIZE;
    }
    else
    {
        dwResult = (DWORD)Stat.st_size;
    }
    return dwResult;
}

/*--------------------------*/
void CloseFile( HANDLE hFile )
{
    close((int)hFile);
}
/*--------------------------*/
void DeleteFile( LPCTSTR szFilename )
{
    remove( szFilename );
}

/*******************************************************************************
*
* Others
*
*/
DWORD WINAPI GetFullPathNameA(LPCTSTR lpFileName,DWORD dwBufferLength,LPTSTR lpBuffer,LPTSTR *lpFilePart)
{
int nStrlen;

    strncpy( lpBuffer, lpFileName, dwBufferLength );
    if( lpFilePart != NULL )
    {
        *lpFilePart = lpBuffer;
    }
    nStrlen = strlen( lpFileName );
    if( (int)dwBufferLength <= nStrlen )
    {
        return (DWORD)(nStrlen + 1);
    }
    else
    {
        return (DWORD)nStrlen;
    }
}
DWORD WINAPI GetFullPathNameW(const WCHAR* lpFileName,DWORD nBufferLength,LPTSTR lpBuffer,LPTSTR *lpFilePart)
{
    return GetFullPathNameA( (LPCTSTR)lpFileName, nBufferLength, lpBuffer, lpFilePart );
}

extern "C" void RTE_SUCCESS(void)
{
    printf( "RTE_SUCCESS is called, who is the caller?????\n" );
}

/* end of file */
